package main

import (
	"fmt"
	"sync"

	"github.com/kjk/u"
	"github.com/rodrigocfd/windigo/win"
	"github.com/rodrigocfd/windigo/win/co"
	"github.com/wailsapp/wails/v2/pkg/runtime"
)

const kPipeName = "\\\\.\\pipe\\LOCAL\\ArsLexis-Logger"
const kBufSize = 1024 * 16
const PIPE_UNLIMITED_INSTANCES = 255
const INVALID_HANDLE_VALUE = -1

var connNo = 1

var wc sync.WaitGroup

func IsValidHandle(h win.HANDLE) bool {
	invalid := h == 0 || int(h) == INVALID_HANDLE_VALUE
	return !invalid
}

func handlePipe(hPipe win.HPIPE, no int) {
	ctx := gApp.ctx
	var buf [kBufSize]byte
	var cbBytesRead = 0
	for {
		n, err := hPipe.ReadFile(buf[:], nil)
		if err != nil {
			s := fmt.Sprintf("ReadFile: returned %s\n", err)
			runtime.EventsEmit(ctx, "plog", s, no)
			break
		}
		cbBytesRead += int(n)
		d := buf[:int(n)]
		runtime.EventsEmit(ctx, "plog", string(d), no)
	}
	hPipe.DisconnectNamedPipe()
	hPipe.CloseHandle()
	wc.Done()
}

func createNamedPipe() (win.HPIPE, error) {
	const openMode = co.PIPE_ACCESS_INBOUND
	const mode = co.PIPE_TYPE_MESSAGE | co.PIPE_READMODE_MESSAGE | co.PIPE_WAIT
	const maxInstances = PIPE_UNLIMITED_INSTANCES
	return win.CreateNamedPipe(kPipeName, openMode, mode, maxInstances, kBufSize, kBufSize, 0, nil)
}

func pipeThread() {
	var hPipe win.HPIPE
	var err error
	for {
		hPipe, err = createNamedPipe()
		u.Must(err)
		if !IsValidHandle(win.HANDLE(hPipe)) {
			return
		}
		err = hPipe.ConnectNamedPipe()
		if err != nil {
			fmt.Printf("client couldn't connect to our pipe\n")
			hPipe.CloseHandle()
			continue
		}
		wc.Add(1)
		go handlePipe(hPipe, connNo)
		connNo++
	}
}
